import React, { useEffect, RefObject, useRef } from 'react';
import * as d3 from 'd3';

const TimeSeries = (props : any) => {
  const ref: RefObject<HTMLDivElement> = React.createRef();
  const drawn = useRef(false); // nouvelle référence pour stocker l'état de l'objet

  useEffect(() => {
    if (!drawn.current) { // vérifie si l'objet a déjà été dessiné
      draw();
      drawn.current = true; // met à jour l'état de l'objet
    }
  }, []);

  let data  :{ date: Date, value: number }[] = [];

  if (typeof props.fetchedData === "undefined" ){
    data = [
      { date: new Date('2020-01-01'), value: 25 },
      { date: new Date('2020-02-01'), value: 37 },
      { date: new Date('2020-03-01'), value: 45 },
    ];
  }else{
    let fetchedData = props.fetchedData["hydra:member"];
    console.log("fetched : " + JSON.stringify(props.fetchedData["hydra:member"].length));
    for (let index = 0; index < fetchedData.length; index++) {
      const element = {date : new Date(fetchedData[index].date), value : Math.round(fetchedData[index].prix)};
      console.log(typeof fetchedData[index].date)
      data.push(element);
    }
  }
  console.log(data);

  const draw = () => {
    const margin = { top: 20, right: 20, bottom: 30, left: 50 };
    const width = 800 - margin.left - margin.right;
    const height = 400 - margin.top - margin.bottom;

    const container = ref.current;
    const svg = d3.select(container)
      .append('svg')
      .attr('width', width + margin.left + margin.right)
      .attr('height', height + margin.top + margin.bottom)
      .append('g')
      .attr('transform', `translate(${margin.left}, ${margin.top})`);

    const xScale = d3.scaleTime()
      .range([0, width])
      .domain(d3.extent(data, (d) => d.date));

    const yScale = d3.scaleLinear()
      .range([height, 0])
      .domain([0, d3.max(data, (d) => d.value)]);

    const line = d3.line<{ date: Date, value: number }>()
      .x((d) => xScale(d.date))
      .y((d) => yScale(d.value));

    svg.append('path')
      .datum(data)
      .attr('fill', 'none')
      .attr('stroke', 'lightgreen')
      .attr('stroke-width', 1.5)
      .attr('d', line);

    const xAxis = d3.axisBottom(xScale);
    const yAxis = d3.axisLeft(yScale);

    svg.append('g')
      .attr('transform', `translate(0, ${height})`)
      .call(xAxis);

    svg.append('g')
      .call(yAxis);

      const g = svg.append('g');

      const rect = svg.append('rect')
        .attr('width', width)
        .attr('height', height)
        .style('fill', 'none')
        .style('pointer-events', 'all');

        const dateFormat = d3.timeFormat('%B, %Y'); // formats the date as a string

        const verticalLine = g.append('line')
          .attr('x1', 0)
          .attr('y1', 0)
          .attr('x2', 0)
          .attr('y2', height)
          .style('stroke', 'darkgreen')
          .style('opacity', 0); // initially hide the vertical line

        const horizontalLine = g.append('line')
          .attr('x1', 0)
          .attr('y1', 0)
          .attr('x2', width)
          .attr('y2', 0)
          .style('stroke', '#000')
          .style('opacity', 0); // initially hide the horizontal line

        rect.on('mousemove', () => {
          const [x, y] = d3.pointer(event);
          const date = xScale.invert(x);
          const value = yScale.invert(y);
          // Use the bisector function to find the index of the element in the data array that is closest to the mouse cursor
          const bisectDate = d3.bisector((d: any) => d.date).left;
          const i = bisectDate(data, date, 1);
          const dataPoint = data[i];

          // Display the data point if it exists
          if (dataPoint) {
            g.selectAll('text').remove();
            g.append('text')
              .attr('x', xScale(dataPoint.date) + 5)
              .attr('y', yScale(dataPoint.value) - 5)
              .text(`${dateFormat(dataPoint.date)}: ${(dataPoint.value)}€/m²`);

            // update the position of the vertical line
            verticalLine
              .attr('x1', xScale(dataPoint.date))
              .attr('x2', xScale(dataPoint.date))
              .attr('y1',yScale(dataPoint.value))
              .style('opacity', 1); // show the vertical line

            // update the position of the horizontal line
            horizontalLine
              .attr('x2', xScale(dataPoint.date))
              .attr('y1', yScale(dataPoint.value))
              .attr('y2', yScale(dataPoint.value))
              .style('opacity', 1); // show the horizontal line
          } else {
            verticalLine.style('opacity', 0); // hide the vertical line
            horizontalLine.style('opacity', 0);//hide the horizontal line
          }
        });



  };

  return (
    <div className="TimeSeries" ref={ref}>
    </div>
  );
};

export default TimeSeries;
